@@ -14,40 +14,36 @@
 # ==============================================================================
 """Various classes representing distributed values for PS."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import contextlib
 import copy
+import functools
 import threading
 import weakref
 
-import numpy as np
-
 from tensorflow.python.distribute import distribute_lib
 from tensorflow.python.distribute import distribute_utils
-from tensorflow.python.distribute import distribution_strategy_context as ds_context
 from tensorflow.python.distribute import values
 from tensorflow.python.distribute import values_util
 from tensorflow.python.distribute.coordinator import coordinator_context
 from tensorflow.python.eager import context
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops
-from tensorflow.python.framework import tensor_spec
+from tensorflow.python.framework import tensor
+from tensorflow.python.framework import tensor_conversion_registry
 from tensorflow.python.ops import array_ops
+from tensorflow.python.ops import handle_data_util
 from tensorflow.python.ops import lookup_ops
 from tensorflow.python.ops import resource_variable_ops
 from tensorflow.python.ops import variable_scope as vs
 from tensorflow.python.saved_model import save_context
-from tensorflow.python.training.tracking import base as trackable
+from tensorflow.python.trackable import base as trackable
 from tensorflow.python.types import core
-from tensorflow.python.util.lazy_loader import LazyLoader
+from tensorflow.python.util import numpy_compat
 
-load_context = LazyLoader(
-    "load_context", globals(),
-    "tensorflow.python.keras.saving.saved_model.load_context"
-)
+
+TRACKABLE_RESOURCE_METHODS = [
+    "_create_resource", "_initialize", "_destroy_resource"
+]
 
 
 # Variable used in PSStrategy TF 1, TF2 and CentralStorageStrategy.
@@ -81,7 +77,7 @@ class AggregatingVariable(resource_variable_ops.BaseResourceVariable,
     Raises:
       RuntimeError: If trying to deepcopy into a different strategy.
     """
-    with ds_context.enter_or_assert_strategy(self._distribute_strategy):
+    with distribute_lib.enter_or_assert_strategy(self._distribute_strategy):
       v = copy.deepcopy(self._v, memo)
 
     copied_variable = type(self)(
@@ -104,9 +100,9 @@ class AggregatingVariable(resource_variable_ops.BaseResourceVariable,
     return getattr(self._v, name)
 
   def _assign_func(self, *args, **kwargs):
-    with ds_context.enter_or_assert_strategy(self._distribute_strategy):
+    with distribute_lib.enter_or_assert_strategy(self._distribute_strategy):
       f = kwargs.pop("f")
-      if ds_context.in_cross_replica_context():
+      if distribute_lib.in_cross_replica_context():
         if distribute_lib.get_update_replica_id() is not None:
           # We are calling an assign function in an update context.
           return f(self._v, *args, **kwargs)
@@ -116,7 +112,7 @@ class AggregatingVariable(resource_variable_ops.BaseResourceVariable,
         return self._distribute_strategy.extended.update(
             self, f, args=args, kwargs=kwargs)
       else:
-        replica_context = ds_context.get_replica_context()
+        replica_context = distribute_lib.get_replica_context()
         assert replica_context
         # We are calling an assign function in replica context.
         # We reduce the value we want to assign/add/sub. More details about how
@@ -171,7 +167,7 @@ class AggregatingVariable(resource_variable_ops.BaseResourceVariable,
     return self._v.initial_value
 
   @property
-  def op(self):
+  def op(self) -> ops.Operation:
     return self._v.op
 
   def value(self):
@@ -224,13 +220,28 @@ class AggregatingVariable(resource_variable_ops.BaseResourceVariable,
       return self._v._gather_saveables_for_checkpoint()  # pylint:disable=protected-access
     return {trackable.VARIABLE_VALUE_KEY: self._v}
 
-  def _map_resources(self, save_options):
+  def _export_to_saved_model_graph(self, object_map, tensor_map,
+                                   options, **kwargs):
     """For implementing `Trackable`."""
     # By delegating this method to the wrapped variable, SavedModel with
     # AggregatingVariable are identical to SavedModel with normal variables.
-    obj_map, resource_map = self._v._map_resources(save_options)  # pylint:disable=protected-access
-    obj_map[self] = obj_map[self._v]
-    return obj_map, resource_map
+    resource_list = self._v._export_to_saved_model_graph(object_map, tensor_map,  # pylint:disable=protected-access
+                                                         options, **kwargs)
+    object_map[self] = object_map[self._v]
+    return resource_list
+
+  def _copy_trackable_to_cpu(self, object_map):
+    """For implementing `Trackable`."""
+    # Create a copy of `self._v` to object_map, then create a new copy of self
+    # that wraps the copy of `self._v`.
+    # When updating value, only the lowest-level variable will actually do that,
+    # the copy of `AggregatingVariable` is more like a shell.
+    self._v._copy_trackable_to_cpu(object_map)  # pylint:disable=protected-access
+    if self not in object_map:
+      # If copy of `self` not populated yet, initialize one.
+      object_map[self] = AggregatingVariable(self._distribute_strategy,
+                                             object_map[self._v],
+                                             self._aggregation)
 
   # pylint: disable=multiple-statements
   def __add__(self, o):
@@ -415,7 +426,7 @@ class CachingVariable(resource_variable_ops.BaseResourceVariable, core.Tensor):
     return self._v.initial_value
 
   @property
-  def op(self):
+  def op(self) -> ops.Operation:
     return self._v.op
 
   def value(self):
@@ -459,7 +470,7 @@ class CachingVariable(resource_variable_ops.BaseResourceVariable, core.Tensor):
     return self._v.constraint
 
   def __array__(self, dtype=None):
-    return np.asarray(self.numpy(), dtype=dtype)
+    return numpy_compat.np_asarray(self.numpy(), dtype=dtype)
 
   def __complex__(self):
     return complex(self.value().numpy())
@@ -495,7 +506,7 @@ class CachingVariable(resource_variable_ops.BaseResourceVariable, core.Tensor):
   @classmethod
   def _overload_overloadable_operators(cls):
     """Register overloads for all operators."""
-    for operator in ops.Tensor.OVERLOADABLE_OPERATORS:
+    for operator in tensor.Tensor.OVERLOADABLE_OPERATORS:
       # Overloading __eq__ or __ne__ does not work as expected.
       if operator == "__eq__" or operator == "__ne__":
         continue
@@ -503,8 +514,8 @@ class CachingVariable(resource_variable_ops.BaseResourceVariable, core.Tensor):
 
   @classmethod
   def _tensor_overload_operator(cls, operator):
-    """Delegate an operator overload to `ops.Tensor`."""
-    tensor_operator = getattr(ops.Tensor, operator)
+    """Delegate an operator overload to `tensor.Tensor`."""
+    tensor_operator = getattr(tensor.Tensor, operator)
 
     def _operator(v, *args, **kwargs):
       return tensor_operator(v.value(), *args, **kwargs)  # pylint: disable=protected-access
@@ -513,13 +524,26 @@ class CachingVariable(resource_variable_ops.BaseResourceVariable, core.Tensor):
   def _gather_saveables_for_checkpoint(self):
     return {trackable.VARIABLE_VALUE_KEY: self._v}
 
-  def _map_resources(self, save_options):
+  def _export_to_saved_model_graph(self, object_map, tensor_map,
+                                   options, **kwargs):
     """For implementing `Trackable`."""
     # By delegating this method to the wrapped variable, SavedModel with
     # AggregatingVariable are identical to SavedModel with normal variables.
-    obj_map, resource_map = self._v._map_resources(save_options)  # pylint:disable=protected-access
-    obj_map[self] = obj_map[self._v]
-    return obj_map, resource_map
+    resource_list = self._v._export_to_saved_model_graph(object_map, tensor_map,  # pylint:disable=protected-access
+                                                         options, **kwargs)
+    object_map[self] = object_map[self._v]
+    return resource_list
+
+  def _copy_trackable_to_cpu(self, object_map):
+    """For implementing `Trackable`."""
+    # Create a copy of `self._v` to object_map, then create a new copy of self
+    # that wraps the copy of `self._v`.
+    # When updating value, only the lowest-level variable will actually do that,
+    # the copy of `CachingVariable` is more like a shell.
+    self._v._copy_trackable_to_cpu(object_map)  # pylint:disable=protected-access
+    if self not in object_map:
+      # If copy of `self` not populated yet, initialize one.
+      object_map[self] = CachingVariable(object_map[self._v])
 
 
 # Register a conversion function which reads the value of the variable,
@@ -528,8 +552,8 @@ def _tensor_conversion_aggregate(var, dtype=None, name=None, as_ref=False):
   return var._dense_var_to_tensor(dtype, name, as_ref)  # pylint: disable=protected-access
 
 
-ops.register_tensor_conversion_function(AggregatingVariable,
-                                        _tensor_conversion_aggregate)
+tensor_conversion_registry.register_tensor_conversion_function(
+    AggregatingVariable, _tensor_conversion_aggregate)
 
 
 # Register a conversion function which reads the value of the variable,
@@ -538,12 +562,133 @@ def _tensor_conversion_caching(var, dtype=None, name=None, as_ref=False):
   return var._dense_var_to_tensor(dtype, name, as_ref)  # pylint: disable=protected-access
 
 
-ops.register_tensor_conversion_function(CachingVariable,
-                                        _tensor_conversion_caching)
+tensor_conversion_registry.register_tensor_conversion_function(
+    CachingVariable, _tensor_conversion_caching)
 
 CachingVariable._overload_overloadable_operators()  # pylint: disable=protected-access
 
 
+class PerWorkerVariable(resource_variable_ops.BaseResourceVariable):
+  """A wrapper around unsynced variables created on workers.
+
+  `PerWorkerVariable`s are variables that are stored on workers and not
+  synchronized. A `PerWorkerVariable` is really a wrapper around multiple
+  independent `Variable`s stored on independent worker machines. 
+  `PerWorkerVariable` is currently only tested and supported when used with
+  `ParameterServerStrategy`. A `PerWorkerVariable` can be created by creating a
+  `Variable` within strategy scope and using the `per_worker_variable` flag,
+  e.g.:
+
+  ```
+  with strategy.scope():
+    var = tf.Variable(initial_value=0.0, per_worker_variable=True)
+  ```
+
+  The implementation modifies the graph to ensure that a worker's local version
+  of the variable is used for computation at call time, while needing only one
+  function trace and requiring no code changes beyond the `per_worker_variable`
+  flag. `PerWorkerVariable`s can thus be treated like a standard `Variable`, but
+  support is experimental and not all ops have been tested.
+
+  All per-worker values can be retrieved and read into a list via
+  `PerWorkerVariable.read_all()`.
+
+  Caveats:
+    - `PerWorkerVariable`s should not be used as direct inputs to a
+      `tf.function`. That is, they should not appear in a tf.function header as
+      an input argument. However they can still be read and manipulated in a
+      `tf.function`.
+    - The `shape` argument must be fully-defined (no `None` entries) or left
+      empty. Partially-defined shapes are not yet supported.
+    - Automatic control dependencies do not work with `PerWorkerVariable`s, so
+      returning a `PerWorkerVariable` is not supported, and `read_all()` should 
+      be used to retrieve values. (TODO: b/286052052)
+    - `PerWorkerVariable`s should not be created within a `tf.function`.
+  """
+
+  def __init__(self, strategy, next_creator, **kwargs):
+    self._coordinator = strategy._cluster_coordinator
+    self._per_worker_vars = None
+    self._var_creator = functools.partial(next_creator, **kwargs)
+
+    self._coordinator_instance = next_creator(**kwargs)
+
+    # Set ResourceVariable attributes based on kwargs
+    if kwargs.get("in_graph_mode") is None:
+      with ops.init_scope():
+        self._in_graph_mode = not context.executing_eagerly()
+    else:
+      self._in_graph_mode = kwargs["in_graph_mode"]
+
+    self._cached_value = None
+    self._shape = self._coordinator_instance.shape
+    self._dtype = self._coordinator_instance.dtype
+    self._trainable = False  # not supported
+    self._unique_id = kwargs.get("unique_id")
+    if kwargs.get("handle_name") is None:
+      self._handle_name = "Variable:0"
+    else:
+      self._handle_name = kwargs["handle_name"] + ":0"
+    self._validate_shape = kwargs.get("validate_shape", True)
+
+  @classmethod
+  def _variable_call(cls, *args, **kwargs):
+    """Override to be a no-op to avoid metaclass creating ResourceVariables."""
+    return None
+
+  @property
+  def handle(self):
+    if context.executing_eagerly() or save_context.in_save_context():
+      return self._coordinator_instance.handle
+    else:
+      self._maybe_create_per_worker_vars()
+      closure, spec = self.handle_call_time_value()
+      return ops.get_default_graph().capture_call_time_value(
+          closure,
+          spec)
+
+  def handle_call_time_value(self):
+    """Returns a closure to run for a handle at call time and its spec.
+
+    This function is called in self.handle to create a placeholder
+    which returns a handle on some worker or on the coordinator.
+    """
+
+    def closure():
+      dispatch_context = coordinator_context.get_current_dispatch_context()
+      if dispatch_context:
+        remote_value = self._per_worker_vars._values[  # pylint: disable=protected-access
+            dispatch_context.worker_index]
+        ret = dispatch_context.maybe_get_remote_value(remote_value)
+        return ret.handle
+      else:
+        # Only needed for tracing
+        return self._coordinator_instance.handle
+    return closure, PerWorkerVariableSpec(
+        value=self._coordinator_instance.handle)
+
+  def _maybe_create_per_worker_vars(self):
+    """Create variable on each worker if it hasn't been created."""
+    if not self._per_worker_vars:
+      self._per_worker_vars = (
+          self._coordinator._create_per_worker_variables(self._var_creator))  # pylint: disable=protected-access
+
+  def read_all(self):
+    """Synchronously read variables from all workers into a list of Tensors."""
+    return [wv.get() for wv in self._per_worker_vars._values]  # pylint: disable=protected-access
+
+
+class PerWorkerVariableSpec(tensor.TensorSpec):
+  def __init__(self, value=None, name=None):
+    super().__init__(value.shape, value.dtype, name=name)
+    self._value = value
+
+  def placeholder_value(self, placeholder_context):
+    placeholder = super().placeholder_value(placeholder_context)
+    handle_data_util.set_handle_data(placeholder, self._value._handle_data)  # pylint: disable=protected-access
+    return placeholder
+
+
 class DistributedTable(lookup_ops.StaticHashTable):
   """A distributed StaticHashTable for ParameterServerStrategy.
 
@@ -562,7 +707,8 @@ class DistributedTable(lookup_ops.StaticHashTable):
   """
 
   def __init__(self, strategy, wrapped_creator):
-
+    distribute_lib.distribution_strategy_input_api_counter.get_cell(
+        self.__class__.__name__, "PSSDistributedLookupTable").increase_by(1)
     self._coordinator_instance = wrapped_creator()
     self._wrapped_creator = wrapped_creator
     self._coordinator = strategy._cluster_coordinator
@@ -616,14 +762,13 @@ class DistributedTable(lookup_ops.StaticHashTable):
       if dispatch_context:
         remote_value = self._distributed_table._values[  # pylint: disable=protected-access
             dispatch_context.worker_index]
-        dispatch_context.maybe_rebuild_remote_values(remote_value)
         ret = dispatch_context.maybe_get_remote_value(remote_value)
         return ret
 
       else:
         return self._coordinator_instance.resource_handle
 
-    return closure, tensor_spec.TensorSpec([], dtype=dtypes.resource)
+    return closure, tensor.TensorSpec([], dtype=dtypes.resource)
 
   def _maybe_build_distributed_table(self):
     """Create table objects and resources on each worker if hasn't been created."""
@@ -650,6 +795,21 @@ class DistributedTable(lookup_ops.StaticHashTable):
           spec,
           default_value=self._coordinator_instance.resource_handle)
 
+  @property
+  def is_distributed_table(self):
+    return True
+
+  def __tf_experimental_restore_capture__(
+      self, concrete_function, internal_capture):
+    closure, spec = self.resource_handle_call_time_value()
+    concrete_function.graph.replace_capture_with_deferred_capture(
+        self._coordinator_instance.resource_handle,
+        closure,
+        spec,
+        default_value=self._coordinator_instance.resource_handle,
+        placeholder=internal_capture)
+    return concrete_function.graph.deferred_external_captures[-1]
+
 
 _local_resource_restore_context = threading.local()
 
@@ -689,6 +849,11 @@ class LocalResourceRestoreContext(object):
 class RestoredDistributedTable(DistributedTable):
   """A restored and distributed StaticHashTable for ParameterServerStrategy."""
 
+  def __init__(self, strategy, wrapped_creator):
+    # Wait for all resource functions to have been set before building the table
+    self._has_resource_functions = threading.Condition()
+    super().__init__(strategy, wrapped_creator)
+
   def resource_handle_call_time_value(self):
     """Returns a closure to run for a resource handle at call time and its spec.
 
@@ -722,7 +887,6 @@ class RestoredDistributedTable(DistributedTable):
           remote_value = self._distributed_table._values[  # pylint: disable=protected-access
               dispatch_context.worker_index]
 
-        dispatch_context.maybe_rebuild_remote_values(remote_value)
         ret = dispatch_context.maybe_get_remote_value(remote_value)
         return ret
 
@@ -730,10 +894,10 @@ class RestoredDistributedTable(DistributedTable):
 
         return self._coordinator_instance.resource_handle
 
-    return closure, tensor_spec.TensorSpec(shape=(), dtype=dtypes.resource)
+    return closure, tensor.TensorSpec(shape=(), dtype=dtypes.resource)
 
   def __setattr__(self, name, value):
-    if name in ["_create_resource", "_initialize", "_destroy_resource"]:
+    if name in TRACKABLE_RESOURCE_METHODS:
       # When a StaticHashTable is loaded with `tf.saved_model.load`, it becomes
       # a RestoredResource with dummy `_create_resource`, `_initialize`, and
       # `_destroy_resource" methods. Similarly, when loaded with
@@ -746,12 +910,13 @@ class RestoredDistributedTable(DistributedTable):
       # been created. We store them in '_restored_function' and set them to the
       # distributed tables when they're created in
       # `self._maybe_build_distributed_table.create_copy`.
-      if load_context.in_load_context or ("RestoredStaticHashtable"
-                                          in self._wrapped.__class__.__name__):
-
-        if not hasattr(self, "_restored_function"):
-          self._restored_function = {}
-        self._restored_function[name] = value
+      if not hasattr(self, "_restored_function"):
+        self._restored_function = {}
+      self._restored_function[name] = value
+      if all(method in self._restored_function
+             for method in TRACKABLE_RESOURCE_METHODS):
+        with self._has_resource_functions:
+          self._has_resource_functions.notify_all()
       return self._coordinator_instance.__setattr__(name, value)
     else:
       return super(RestoredDistributedTable, self).__setattr__(name, value)
@@ -775,6 +940,13 @@ class RestoredDistributedTable(DistributedTable):
 
         def create_copy():
           new_table = self._wrapped_creator()
+          # Wait until all resource functions are available before setting them
+          # on new_table.
+          with self._has_resource_functions:
+            while not hasattr(self, "_restored_function") or any(
+                method not in self._restored_function
+                for method in TRACKABLE_RESOURCE_METHODS):
+              self._has_resource_functions.wait()
 
           if hasattr(self, "_restored_function"):
             with with_local_resource_restore_context(new_table):
