@@ -26,6 +26,7 @@ from tensorflow.python.framework import composite_tensor_gradient
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import indexed_slices
 from tensorflow.python.framework import ops
+from tensorflow.python.framework import tensor as tensor_lib
 from tensorflow.python.framework import tensor_shape
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import control_flow_ops
@@ -64,8 +65,13 @@ def _MarkReachedOps(from_ops, reached_ops, func_graphs):
           queue.extend(_Consumers(output, func_graphs))
 
 
-def _PendingCount(to_ops, from_ops, colocate_gradients_with_ops, func_graphs,
-                  xs_set):
+def _PendingCount(
+    to_ops: list[ops.Operation],
+    from_ops: list[ops.Operation],
+    colocate_gradients_with_ops,
+    func_graphs,
+    xs_set,
+):
   """Initialize the pending count for ops between two lists of Operations.
 
   'pending_count[op]' indicates the number of backprop inputs
@@ -166,63 +172,84 @@ def _DefaultGradYs(grad_ys,
         if y.dtype.is_complex:
           raise TypeError(
               f"Gradients of complex tensors ({y}) must set grad_ys (y.dtype = "
-              f"{dtypes.as_dtype(y.dtype).name})")
+              f"{dtypes.as_dtype(y.dtype).name})"
+          )
         new_grad_ys.append(
             array_ops.ones(
-                array_ops.shape(y), dtype=y.dtype, name="grad_ys_%d" % i))
+                array_ops.shape(y), dtype=y.dtype, name="grad_ys_%d" % i
+            )
+        )
         continue
       if y.dtype.is_floating or y.dtype.is_integer:
         if not grad_y.dtype.is_floating and not grad_y.dtype.is_integer:
           raise TypeError(
               f"Gradient type {dtypes.as_dtype(grad_y.dtype).name} generated "
               f"for real or integer-valued tensor {y} with type "
-              f"{dtypes.as_dtype(y.dtype).name} must be real or integer")
+              f"{dtypes.as_dtype(y.dtype).name} must be real or integer"
+          )
       elif y.dtype.is_complex:
         if not grad_y.dtype.is_complex:
           raise TypeError(
               f"Gradient type {dtypes.as_dtype(grad_y.dtype).name} generated "
               f"for complex-valued tensor {y} with type "
-              f"{dtypes.as_dtype(y.dtype).name} must be real")
+              f"{dtypes.as_dtype(y.dtype).name} must be real"
+          )
       elif y.dtype == dtypes.variant:
         if grad_y.dtype != dtypes.variant:
           raise TypeError(
               f"Gradient type {dtypes.as_dtype(grad_y.dtype).name} generated "
               f"for variant tensor {y} with type "
-              f"{dtypes.as_dtype(y.dtype).name} must be variant")
+              f"{dtypes.as_dtype(y.dtype).name} must be variant"
+          )
       elif y.dtype == dtypes.resource:
         # We assume y is the handle of a ResourceVariable. The gradient of a
         # ResourceVariable should be a numeric value, not another resource.
         if grad_y.dtype == dtypes.resource:
-          raise TypeError(f"Input gradient {grad_y} for resource tensor {y} "
-                          "should not be a resource")
+          raise TypeError(
+              f"Input gradient {grad_y} for resource tensor {y} "
+              "should not be a resource"
+          )
       else:
         raise TypeError(
             f"Tensor {y} with type {dtypes.as_dtype(y.dtype).name} must be "
-            "numeric to obtain a default gradient")
+            "numeric to obtain a default gradient"
+        )
       # Create a grad_y tensor in the name scope of the gradient.
       # Required for TensorArrays to identify which gradient call a
       # grad_y value is coming from.
       if isinstance(grad_y, indexed_slices.IndexedSlices):
         new_grad_ys.append(
             indexed_slices.IndexedSlices(
-                indices=(array_ops.identity(
-                    grad_y.indices, name="grad_ys_%d_indices" % i)
-                         if isinstance(grad_y.indices, ops.Tensor) else
-                         grad_y.indices),
-                values=(array_ops.identity(
-                    grad_y.values, name="grad_ys_%d_values" % i) if isinstance(
-                        grad_y.values, ops.Tensor) else grad_y.values),
-                dense_shape=(array_ops.identity(
-                    grad_y.dense_shape, name="grad_ys_%d_shape" % i)
-                             if isinstance(grad_y.dense_shape, ops.Tensor) else
-                             grad_y.dense_shape)))
+                indices=(
+                    array_ops.identity(
+                        grad_y.indices, name="grad_ys_%d_indices" % i
+                    )
+                    if isinstance(grad_y.indices, tensor_lib.Tensor)
+                    else grad_y.indices
+                ),
+                values=(
+                    array_ops.identity(
+                        grad_y.values, name="grad_ys_%d_values" % i
+                    )
+                    if isinstance(grad_y.values, tensor_lib.Tensor)
+                    else grad_y.values
+                ),
+                dense_shape=(
+                    array_ops.identity(
+                        grad_y.dense_shape, name="grad_ys_%d_shape" % i
+                    )
+                    if isinstance(grad_y.dense_shape, tensor_lib.Tensor)
+                    else grad_y.dense_shape
+                ),
+            )
+        )
       else:
         new_grad_ys.append(array_ops.identity(grad_y, name="grad_ys_%d" % i))
 
   return new_grad_ys
 
 
-def _VerifyGeneratedGradients(grads, op):
+def _VerifyGeneratedGradients(grads, op: ops.Operation):
   """Verify that gradients are valid in number and type.
 
   Args:
@@ -239,11 +266,18 @@ def _VerifyGeneratedGradients(grads, op):
     return
 
   if len(grads) != len(op.inputs):
-    raise ValueError(f"Num gradients {len(grads)} generated for op "
-                     f"{op.node_def} do not match num inputs {len(op.inputs)}")
+    raise ValueError(
+        f"Num gradients {len(grads)} generated for op "
+        f"{op.node_def} do not match num inputs {len(op.inputs)}"
+    )
 
 
-def _StopOps(from_ops, stop_gradient_ops, pending_count, xs_set):
+def _StopOps(
+    from_ops: list[ops.Operation],
+    stop_gradient_ops: list[ops.Operation],
+    pending_count,
+    xs_set,
+):
   """The set of ops that terminate the gradient computation.
 
   This computes the frontier of the forward graph *before* which backprop
@@ -278,7 +312,11 @@ def _StopOps(from_ops, stop_gradient_ops, pending_count, xs_set):
 
 
 @contextlib.contextmanager
-def _maybe_colocate_with(op, gradient_uid, colocate_gradients_with_ops):  # pylint: disable=invalid-name
+def _maybe_colocate_with(  # pylint: disable=invalid-name
+    op: ops.Operation,
+    gradient_uid,
+    colocate_gradients_with_ops,
+):
   """Context to colocate with `op` if `colocate_gradients_with_ops`."""
   if colocate_gradients_with_ops:
     with ops._colocate_with_for_gradient(op, gradient_uid):  # pylint: disable=protected-access
@@ -287,11 +325,11 @@ def _maybe_colocate_with(op, gradient_uid, colocate_gradients_with_ops):  # pyli
     yield
 
 
-def _IsPartitionedCall(op):
+def _IsPartitionedCall(op: ops.Operation):
   return op.type == "PartitionedCall" or op.type == "StatefulPartitionedCall"
 
 
-def _SymGrad(op, out_grads):
+def _SymGrad(op: ops.Operation, out_grads):
   """Backprop through a function call node op given its outputs' gradients."""
   f_in = [x for x in op.inputs] + out_grads
   f_types = [default_gradient.get_zeros_dtype(x) for x in op.inputs]
@@ -306,7 +344,7 @@ def _SymGrad(op, out_grads):
   return in_grads
 
 
-def _MaybeCompile(scope, op, func, grad_fn):
+def _MaybeCompile(scope, op: ops.Operation, func, grad_fn):
   """Compile the calculation in grad_fn if op was marked as compiled."""
   scope = scope.rstrip("/").replace("/", "_")
   if func is not None:
@@ -343,7 +381,11 @@ def _MaybeCompile(scope, op, func, grad_fn):
     return grad_fn()
 
 
-def _RaiseNoGradWrtInitialLoopValError(op, from_ops, xs_set):
+def _RaiseNoGradWrtInitialLoopValError(
+    op: ops.Operation,
+    from_ops: list[ops.Operation],
+    xs_set,
+):
   """Raises an error if we backprop through a loop var."""
   # Find the nearest 'to_op' reachable from 'op' to provide a more helpful error
   # message.
@@ -396,7 +438,7 @@ def _MaybeCaptured(t):
   return t
 
 
-def _NonEagerInputs(op, xs_set):
+def _NonEagerInputs(op: ops.Operation, xs_set):
   """Returns the inputs of op, crossing closure boundaries where necessary.
 
   Does not return any captured EagerTensors, i.e., the number of tensors
@@ -415,7 +457,7 @@ def _NonEagerInputs(op, xs_set):
 
 # TODO(skyewm): plumbing xs through everywhere is ugly, consider making
 # _GradientsHelper a class with xs as a member variable.
-def _Inputs(op, xs_set):
+def _Inputs(op: ops.Operation, xs_set):
   """Returns the inputs of op, crossing closure boundaries where necessary.
 
   Args:
@@ -594,10 +636,11 @@ def _GradientsHelper(ys,
         func_call = None
         is_partitioned_call = _IsPartitionedCall(op)
         # pylint: disable=protected-access
-        is_func_call = (
-            src_graph._is_function(op.type) or is_partitioned_call)
+        is_func_call = src_graph._is_function(op.type) or is_partitioned_call
         # pylint: enable=protected-access
-        has_out_grads = any(isinstance(g, ops.Tensor) or g for g in out_grads)
+        has_out_grads = any(
+            isinstance(g, tensor_lib.Tensor) or g for g in out_grads
+        )
         if has_out_grads and (op not in stop_ops):
           try:
             grad_fn = ops.get_gradient_function(op)
@@ -662,9 +705,12 @@ def _GradientsHelper(ys,
           # output, it means that the cost does not depend on output[i],
           # therefore dC/doutput[i] is 0.
           for i, out_grad in enumerate(out_grads):
-            if (not isinstance(out_grad, ops.Tensor) and not out_grad) and (
+            if (
+                not isinstance(out_grad, tensor_lib.Tensor) and not out_grad
+            ) and (
                 (not grad_fn and is_func_call)
-                or backprop_util.IsTrainable(op.outputs[i])):
+                or backprop_util.IsTrainable(op.outputs[i])
+            ):
               # Only trainable outputs or outputs for a function call that
               # will use SymbolicGradient get a zero gradient. Gradient
               # functions should ignore the gradient for other outputs.
@@ -710,7 +756,7 @@ def _GradientsHelper(ys,
         # line up with in_grads.
         for i, (t_in, in_grad) in enumerate(zip(_Inputs(op, xs_set), in_grads)):
           if in_grad is not None:
-            if (isinstance(in_grad, ops.Tensor) and
+            if (isinstance(in_grad, tensor_lib.Tensor) and
                 t_in.dtype != dtypes.resource):
               try:
                 in_grad.set_shape(t_in.get_shape())
@@ -734,11 +780,11 @@ def _GradientsHelper(ys,
   return [_GetGrad(grads, x, unconnected_gradients) for x in xs]
 
 
-def _HasAnyNotNoneGrads(grads, op):
+def _HasAnyNotNoneGrads(grads, op: ops.Operation):
   """Return true iff op has real gradient."""
   out_grads = _GetGrads(grads, op)
   for out_grad in out_grads:
-    if isinstance(out_grad, (ops.Tensor, indexed_slices.IndexedSlices)):
+    if isinstance(out_grad, (tensor_lib.Tensor, indexed_slices.IndexedSlices)):
       return True
     if out_grad and isinstance(out_grad, collections_abc.Sequence):
       if any(g is not None for g in out_grad):
@@ -746,12 +792,13 @@ def _HasAnyNotNoneGrads(grads, op):
   return False
 
 
-def _UpdatePendingAndEnqueueReady(grads, op, queue, pending_count, loop_state,
-                                  xs_set):
+def _UpdatePendingAndEnqueueReady(
+    grads, op: ops.Operation, queue, pending_count, loop_state, xs_set
+):
   """Update pending count for the inputs of op and enqueue ready ops."""
   for x in _NonEagerInputs(op, xs_set):
     pending_count[x.op] -= 1
-    ready = (pending_count[x.op] == 0)
+    ready = pending_count[x.op] == 0
     if loop_state and not ready:
       ready = pending_count[x.op] > 0 and control_flow_util.IsLoopSwitch(x.op)
     if ready:
@@ -831,7 +878,7 @@ def _GetGrad(grads, t, unconnected_gradients):
   return t_grad
 
 
-def _GetGrads(grads, op):
+def _GetGrads(grads, op: ops.Operation):
   """Gets all gradients for op."""
   if op in grads:
     return grads[op]
@@ -842,12 +889,12 @@ def _GetGrads(grads, op):
 def _AccumulatorShape(inputs):
   shape = tensor_shape.unknown_shape()
   for i in inputs:
-    if isinstance(i, ops.Tensor):
+    if isinstance(i, tensor_lib.Tensor):
       shape = shape.merge_with(i.get_shape())
   return shape
 
 
-def _LogOpGradients(op, out_grads, in_grads):
+def _LogOpGradients(op: ops.Operation, out_grads, in_grads):
   """Log the in and out grads of an op."""
   logging.vlog(1, "Gradient for '" + op.name + "'")
 
@@ -981,12 +1028,13 @@ def _AggregatedGrads(grads,
   out_grads = _GetGrads(grads, op)
   for i, out_grad in enumerate(out_grads):
     if loop_state:
-      if isinstance(out_grad, (ops.Tensor, indexed_slices.IndexedSlices)):
+      if isinstance(
+          out_grad, (tensor_lib.Tensor, indexed_slices.IndexedSlices)):
         assert control_flow_util.IsLoopSwitch(op)
         continue
     # Grads have to be Tensors or IndexedSlices
     if (isinstance(out_grad, collections_abc.Sequence) and not all(
-        isinstance(g, (ops.Tensor, indexed_slices.IndexedSlices))
+        isinstance(g, (tensor_lib.Tensor, indexed_slices.IndexedSlices))
         for g in out_grad
         if g is not None)):
       raise TypeError(f"Invalid gradient {out_grad} [index = {i}]. Gradients "
@@ -996,7 +1044,8 @@ def _AggregatedGrads(grads,
       if len(out_grad) < 2:
         used = "nop"
         out_grads[i] = out_grad[0]
-      elif all(isinstance(g, ops.Tensor) for g in out_grad if g is not None):
+      elif all(
+          isinstance(g, tensor_lib.Tensor) for g in out_grad if g is not None):
         tensor_shape = _AccumulatorShape(out_grad)
         if aggregation_method in [
             AggregationMethod.EXPERIMENTAL_TREE,
